(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('redux')) :
  typeof define === 'function' && define.amd ? define(['exports', 'redux'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.ReduxRemember = {}, global.Redux));
})(this, (function (exports, redux) {
  function _typeof(obj) {
    "@babel/helpers - typeof";

    return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) {
      return typeof obj;
    } : function (obj) {
      return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
    }, _typeof(obj);
  }

  function _toPrimitive(input, hint) {
    if (_typeof(input) !== "object" || input === null) return input;
    var prim = input[Symbol.toPrimitive];
    if (prim !== undefined) {
      var res = prim.call(input, hint || "default");
      if (_typeof(res) !== "object") return res;
      throw new TypeError("@@toPrimitive must return a primitive value.");
    }
    return (hint === "string" ? String : Number)(input);
  }

  function _toPropertyKey(arg) {
    var key = _toPrimitive(arg, "string");
    return _typeof(key) === "symbol" ? key : String(key);
  }

  function _defineProperty(obj, key, value) {
    key = _toPropertyKey(key);
    if (key in obj) {
      Object.defineProperty(obj, key, {
        value: value,
        enumerable: true,
        configurable: true,
        writable: true
      });
    } else {
      obj[key] = value;
    }
    return obj;
  }

  function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
    try {
      var info = gen[key](arg);
      var value = info.value;
    } catch (error) {
      reject(error);
      return;
    }
    if (info.done) {
      resolve(value);
    } else {
      Promise.resolve(value).then(_next, _throw);
    }
  }
  function _asyncToGenerator(fn) {
    return function () {
      var self = this,
        args = arguments;
      return new Promise(function (resolve, reject) {
        var gen = fn.apply(self, args);
        function _next(value) {
          asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
        }
        function _throw(err) {
          asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
        }
        _next(undefined);
      });
    };
  }

  var REMEMBER_REHYDRATED = '@@REMEMBER_REHYDRATED';
  var REMEMBER_PERSISTED = '@@REMEMBER_PERSISTED';

  var pick = function pick(src, keys) {
    if (src === null || _typeof(src) !== 'object') {
      return {};
    }
    var index = -1;
    var dest = {};
    while (++index < keys.length) {
      var key = keys[index];
      if (src.hasOwnProperty(key)) {
        dest[key] = src[key];
      }
    }
    return dest;
  };
  var throttle = function throttle(callback, msecs) {
    var nextCallNow = 0;
    var nextCallTimeout;
    return function () {
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }
      var now = +new Date();
      var timeLeft = (nextCallNow || now) - now;
      if (timeLeft <= 0 && !nextCallTimeout) {
        nextCallNow = now + msecs;
        callback.apply(void 0, args);
        return;
      }
      if (nextCallTimeout) {
        clearTimeout(nextCallTimeout);
        nextCallTimeout = null;
      }
      nextCallTimeout = setTimeout(function () {
        nextCallNow = +new Date() + msecs;
        callback.apply(void 0, args);
        clearTimeout(nextCallTimeout);
        nextCallTimeout = null;
      }, timeLeft);
    };
  };
  var debounce = function debounce(callback, msecs) {
    var nextCallTimeout;
    return function () {
      for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
        args[_key2] = arguments[_key2];
      }
      clearTimeout(nextCallTimeout);
      nextCallTimeout = setTimeout(function () {
        callback.apply(void 0, args);
        nextCallTimeout = null;
      }, msecs);
    };
  };

  function ownKeys$1(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
  function _objectSpread$1(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys$1(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys$1(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
  var loadAll = /*#__PURE__*/function () {
    var _ref2 = _asyncToGenerator(function* (_ref) {
      var rememberedKeys = _ref.rememberedKeys,
        driver = _ref.driver,
        prefix = _ref.prefix,
        unserialize = _ref.unserialize;
      var data = yield driver.getItem("".concat(prefix, "rootState"));
      if (data === null || data === undefined) {
        return {};
      }
      return pick(unserialize(data, 'rootState'), rememberedKeys);
    });
    return function loadAll(_x) {
      return _ref2.apply(this, arguments);
    };
  }();
  var loadAllKeyed = /*#__PURE__*/function () {
    var _ref4 = _asyncToGenerator(function* (_ref3) {
      var rememberedKeys = _ref3.rememberedKeys,
        driver = _ref3.driver,
        prefix = _ref3.prefix,
        unserialize = _ref3.unserialize;
      var items = yield Promise.all(rememberedKeys.map(function (key) {
        return driver.getItem("".concat(prefix).concat(key));
      }));
      return rememberedKeys.reduce(function (obj, key, i) {
        if (items[i] !== null && items[i] !== undefined) {
          obj[key] = unserialize(items[i], key);
        }
        return obj;
      }, {});
    });
    return function loadAllKeyed(_x2) {
      return _ref4.apply(this, arguments);
    };
  }();
  var rehydrate = /*#__PURE__*/function () {
    var _ref6 = _asyncToGenerator(function* (store, rememberedKeys, _ref5) {
      var prefix = _ref5.prefix,
        driver = _ref5.driver,
        persistWholeStore = _ref5.persistWholeStore,
        unserialize = _ref5.unserialize;
      var state = store.getState();
      try {
        var load = persistWholeStore ? loadAll : loadAllKeyed;
        state = _objectSpread$1(_objectSpread$1({}, state), yield load({
          rememberedKeys: rememberedKeys,
          driver: driver,
          prefix: prefix,
          unserialize: unserialize
        }));
      } catch (err) {
        console.warn('redux-remember: rehydrate error', err);
      }
      store.dispatch({
        type: REMEMBER_REHYDRATED,
        payload: state
      });
    });
    return function rehydrate(_x3, _x4, _x5) {
      return _ref6.apply(this, arguments);
    };
  }();

  function _arrayWithHoles(arr) {
    if (Array.isArray(arr)) return arr;
  }

  function _iterableToArrayLimit(arr, i) {
    var _i = null == arr ? null : "undefined" != typeof Symbol && arr[Symbol.iterator] || arr["@@iterator"];
    if (null != _i) {
      var _s,
        _e,
        _x,
        _r,
        _arr = [],
        _n = !0,
        _d = !1;
      try {
        if (_x = (_i = _i.call(arr)).next, 0 === i) {
          if (Object(_i) !== _i) return;
          _n = !1;
        } else for (; !(_n = (_s = _x.call(_i)).done) && (_arr.push(_s.value), _arr.length !== i); _n = !0);
      } catch (err) {
        _d = !0, _e = err;
      } finally {
        try {
          if (!_n && null != _i["return"] && (_r = _i["return"](), Object(_r) !== _r)) return;
        } finally {
          if (_d) throw _e;
        }
      }
      return _arr;
    }
  }

  function _arrayLikeToArray$1(arr, len) {
    if (len == null || len > arr.length) len = arr.length;
    for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];
    return arr2;
  }

  function _unsupportedIterableToArray$1(o, minLen) {
    if (!o) return;
    if (typeof o === "string") return _arrayLikeToArray$1(o, minLen);
    var n = Object.prototype.toString.call(o).slice(8, -1);
    if (n === "Object" && o.constructor) n = o.constructor.name;
    if (n === "Map" || n === "Set") return Array.from(o);
    if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray$1(o, minLen);
  }

  function _nonIterableRest() {
    throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
  }

  function _slicedToArray(arr, i) {
    return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray$1(arr, i) || _nonIterableRest();
  }

  function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it["return"] != null) it["return"](); } finally { if (didErr) throw err; } } }; }
  function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }
  function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i]; return arr2; }
  var getObjectType = function getObjectType(o) {
    var _Object$prototype$toS;
    return (_Object$prototype$toS = Object.prototype.toString.call(o).match(/\[object (.*)\]/)) === null || _Object$prototype$toS === void 0 ? void 0 : _Object$prototype$toS[1];
  };
  var isTypedArray = function isTypedArray(o) {
    return /^\[object (?:Float(?:32|64)|(?:Int|Uint)(?:8|16|32)|Uint8Clamped)Array\]$/.test(Object.prototype.toString.call(o));
  };
  var isDeepEqual = function isDeepEqual(a, b) {
    if (a === b) {
      return true;
    }
    if (_typeof(a) !== 'object' || _typeof(b) !== 'object' || a === null || a === undefined || b === null || b === undefined) {
      return a !== a && b !== b;
    }
    if (a.constructor !== b.constructor) {
      return false;
    }
    if (a.constructor === RegExp) {
      return a.source === b.source && a.flags === b.flags;
    }
    if (Array.isArray(a)) {
      if (a.length !== b.length) {
        return false;
      }
      for (var i = a.length; i-- !== 0;) {
        if (!isDeepEqual(a[i], b[i])) {
          return false;
        }
      }
      return true;
    }
    if (isTypedArray(a) && isTypedArray(b)) {
      if (a.byteLength !== b.byteLength) {
        return false;
      }
      for (var _i = a.byteLength; _i-- !== 0;) {
        if (!isDeepEqual(a[_i], b[_i])) {
          return false;
        }
      }
      return true;
    }
    var aType = getObjectType(a);
    var bType = getObjectType(b);
    if (aType === 'DataView' && bType === 'DataView') {
      if (a.byteLength !== b.byteLength || a.byteOffset !== b.byteOffset) {
        return false;
      }
      return isDeepEqual(a.buffer, b.buffer);
    }
    if (aType === 'ArrayBuffer' && bType === 'ArrayBuffer') {
      if (a.byteLength !== b.byteLength) {
        return false;
      }
      return isDeepEqual(new Uint8Array(a), new Uint8Array(b));
    }
    if (aType === 'Map' && bType === 'Map') {
      if (a.size !== b.size) {
        return false;
      }
      var _iterator = _createForOfIteratorHelper(a.entries()),
        _step;
      try {
        for (_iterator.s(); !(_step = _iterator.n()).done;) {
          var _step$value = _slicedToArray(_step.value, 1),
            key = _step$value[0];
          if (!b.has(key)) {
            return false;
          }
        }
      } catch (err) {
        _iterator.e(err);
      } finally {
        _iterator.f();
      }
      var _iterator2 = _createForOfIteratorHelper(a.entries()),
        _step2;
      try {
        for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
          var _step2$value = _slicedToArray(_step2.value, 2),
            _key = _step2$value[0],
            value = _step2$value[1];
          if (!isDeepEqual(value, b.get(_key))) {
            return false;
          }
        }
      } catch (err) {
        _iterator2.e(err);
      } finally {
        _iterator2.f();
      }
      return true;
    }
    if (aType === 'Set' && bType === 'Set') {
      if (a.size !== b.size) {
        return false;
      }
      var _iterator3 = _createForOfIteratorHelper(a.entries()),
        _step3;
      try {
        for (_iterator3.s(); !(_step3 = _iterator3.n()).done;) {
          var _step3$value = _slicedToArray(_step3.value, 1),
            _key2 = _step3$value[0];
          if (!b.has(_key2)) {
            return false;
          }
        }
      } catch (err) {
        _iterator3.e(err);
      } finally {
        _iterator3.f();
      }
      return true;
    }
    if (aType === 'Date' && bType === 'Date') {
      return +a === +b;
    }
    var aKeys = Object.keys(a);
    if (aKeys.length !== Object.keys(b).length) {
      return false;
    }
    for (var _i2 = aKeys.length; _i2-- !== 0;) {
      if (!Object.prototype.hasOwnProperty.call(b, aKeys[_i2])) {
        return false;
      }
    }
    for (var _i3 = aKeys.length; _i3-- !== 0;) {
      var _key3 = aKeys[_i3];
      if (!isDeepEqual(a[_key3], b[_key3])) {
        return false;
      }
    }
    if (aType !== 'Object' && bType !== 'Object') {
      return a === b;
    }
    return true;
  };

  var saveAll = /*#__PURE__*/function () {
    var _ref2 = _asyncToGenerator(function* (state, oldState, _ref) {
      var prefix = _ref.prefix,
        driver = _ref.driver,
        serialize = _ref.serialize;
      if (!isDeepEqual(state, oldState)) {
        return driver.setItem("".concat(prefix, "rootState"), serialize(state, 'rootState'));
      }
    });
    return function saveAll(_x, _x2, _x3) {
      return _ref2.apply(this, arguments);
    };
  }();
  var saveAllKeyed = function saveAllKeyed(state, oldState, _ref3) {
    var prefix = _ref3.prefix,
      driver = _ref3.driver,
      serialize = _ref3.serialize;
    return Promise.all(Object.keys(state).map(function (key) {
      if (isDeepEqual(state[key], oldState[key])) {
        return Promise.resolve();
      }
      return driver.setItem("".concat(prefix).concat(key), serialize(state[key], key));
    }));
  };
  var persist = /*#__PURE__*/function () {
    var _ref4 = _asyncToGenerator(function* () {
      var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var oldState = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      var _ref5 = arguments.length > 2 ? arguments[2] : undefined,
        prefix = _ref5.prefix,
        driver = _ref5.driver,
        persistWholeStore = _ref5.persistWholeStore,
        serialize = _ref5.serialize;
      try {
        var save = persistWholeStore ? saveAll : saveAllKeyed;
        yield save(state, oldState, {
          prefix: prefix,
          driver: driver,
          serialize: serialize
        });
      } catch (err) {
        console.warn('redux-remember: persist error', err);
      }
    });
    return function persist() {
      return _ref4.apply(this, arguments);
    };
  }();

  var init = /*#__PURE__*/function () {
    var _ref2 = _asyncToGenerator(function* (store, rememberedKeys, _ref) {
      var prefix = _ref.prefix,
        driver = _ref.driver,
        serialize = _ref.serialize,
        unserialize = _ref.unserialize,
        persistThrottle = _ref.persistThrottle,
        persistDebounce = _ref.persistDebounce,
        persistWholeStore = _ref.persistWholeStore;
      yield rehydrate(store, rememberedKeys, {
        prefix: prefix,
        driver: driver,
        unserialize: unserialize,
        persistWholeStore: persistWholeStore
      });
      var oldState = {};
      var persistFunc = /*#__PURE__*/function () {
        var _ref3 = _asyncToGenerator(function* () {
          var state = pick(store.getState(), rememberedKeys);
          yield persist(state, oldState, {
            prefix: prefix,
            driver: driver,
            serialize: serialize,
            persistWholeStore: persistWholeStore
          });
          if (!isDeepEqual(state, oldState)) {
            store.dispatch({
              type: REMEMBER_PERSISTED,
              payload: state
            });
          }
          oldState = state;
        });
        return function persistFunc() {
          return _ref3.apply(this, arguments);
        };
      }();
      if (persistDebounce && persistDebounce > 0) {
        store.subscribe(debounce(persistFunc, persistDebounce));
      } else {
        store.subscribe(throttle(persistFunc, persistThrottle));
      }
    });
    return function init(_x, _x2, _x3) {
      return _ref2.apply(this, arguments);
    };
  }();

  function ownKeys(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
  function _objectSpread(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
  var rememberReducer = function rememberReducer(reducer) {
    var data = {
      state: {}
    };
    return function () {
      var _action$type;
      var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : data.state;
      var action = arguments.length > 1 ? arguments[1] : undefined;
      if (action.type && ((action === null || action === void 0 ? void 0 : action.type) === '@@INIT' || action !== null && action !== void 0 && (_action$type = action.type) !== null && _action$type !== void 0 && _action$type.startsWith('@@redux/INIT'))) {
        data.state = _objectSpread({}, state);
      }
      var rootReducer = typeof reducer === 'function' ? reducer : redux.combineReducers(reducer);
      switch (action.type) {
        case REMEMBER_REHYDRATED:
          {
            var rehydratedState = _objectSpread(_objectSpread({}, data.state), action.payload || {});
            data.state = rootReducer(rehydratedState, {
              type: REMEMBER_REHYDRATED,
              payload: rehydratedState
            });
            return data.state;
          }
        default:
          return rootReducer(state, action);
      }
    };
  };
  var rememberEnhancer = function rememberEnhancer(driver, rememberedKeys) {
    var _ref = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {},
      _ref$prefix = _ref.prefix,
      prefix = _ref$prefix === void 0 ? '@@remember-' : _ref$prefix,
      _ref$serialize = _ref.serialize,
      serialize = _ref$serialize === void 0 ? function (data, key) {
        return JSON.stringify(data);
      } : _ref$serialize,
      _ref$unserialize = _ref.unserialize,
      unserialize = _ref$unserialize === void 0 ? function (data, key) {
        return JSON.parse(data);
      } : _ref$unserialize,
      _ref$persistThrottle = _ref.persistThrottle,
      persistThrottle = _ref$persistThrottle === void 0 ? 100 : _ref$persistThrottle,
      persistDebounce = _ref.persistDebounce,
      _ref$persistWholeStor = _ref.persistWholeStore,
      persistWholeStore = _ref$persistWholeStor === void 0 ? false : _ref$persistWholeStor,
      initActionType = _ref.initActionType;
    if (!driver) {
      throw Error('redux-remember error: driver required');
    }
    if (!Array.isArray(rememberedKeys)) {
      throw Error('redux-remember error: rememberedKeys needs to be an array');
    }
    var storeCreator = function storeCreator(createStore) {
      return function (rootReducer, initialState, enhancer) {
        var isInitialized = false;
        var initialize = function initialize(store) {
          return init(store, rememberedKeys, {
            driver: driver,
            prefix: prefix,
            serialize: serialize,
            unserialize: unserialize,
            persistThrottle: persistThrottle,
            persistDebounce: persistDebounce,
            persistWholeStore: persistWholeStore
          });
        };
        var store = createStore(function (state, action) {
          if (!isInitialized && initActionType && action.type === initActionType) {
            isInitialized = true;
            setTimeout(function () {
              return initialize(store);
            }, 0);
          }
          return rootReducer(state, action);
        }, initialState, enhancer);
        if (!initActionType) {
          isInitialized = true;
          initialize(store);
        }
        return store;
      };
    };
    return storeCreator;
  };

  exports.REMEMBER_PERSISTED = REMEMBER_PERSISTED;
  exports.REMEMBER_REHYDRATED = REMEMBER_REHYDRATED;
  exports.rememberEnhancer = rememberEnhancer;
  exports.rememberReducer = rememberReducer;

}));
//# sourceMappingURL=redux-remember.js.map
